==> astrophot-0.11.0-py2.py3-none-any.whl: astrophot/models/airy_psf.py <==
import torch
import numpy as np

from ..utils.decorators import ignore_numpy_warnings, default_internal
from ._shared_methods import select_target
from .star_model_object import Star_Model
from .. import AP_config

__all__ = ("Airy_Star",)


class Airy_Star(Star_Model):
    """The Airy disk is an analytic description of the diffraction pattern
    for a circular aperture.

    The diffraction pattern is described exactly by the configuration
    of the lens system under the assumption that all elements are
    perfect. This expression goes as:

    .. math::

      I(\\theta) = I_0\\left[\\frac{2J_1(x)}{x}\\right]^2

      x = ka\\sin(\\theta) = \\frac{2\\pi a r}{\\lambda R}

    where :math:`I(\\theta)` is the intensity as a function of the
    angular poisition within the diffraction system along its main
    axis, :math:`I_0` is the central intensity of the airy disk,
    :math:`J_1` is the Bessel function of the first kind of order one,
    :math:`k = \\frac{2\\pi}{\\lambda}` is the wavenumber of the
    light, :math:`a` is the aperture radius, :math:`r` is the radial
    position from the center of the pattern, :math:`R` is the distance
    from the circular aperture to the observation plane.

    In the `Airy_Star` class we combine the parameters
    :math:`a,R,\\lambda` into a single ratio to be optimized (or fixed
    by the optical configuration).

    """
    model_type = f"airy {Star_Model.model_type}"
    parameter_specs = {
        "I0": {"units": "log10(flux/arcsec^2)"},
        "aRL": {"units": "a/(R lambda)"}
    }
    _parameter_order = Star_Model._parameter_order + ("I0", "aRL")
    useable = True

    @torch.no_grad()
    @ignore_numpy_warnings
    @select_target
    @default_internal
    def initialize(self, target=None, parameters=None, **kwargs):
        super().initialize(target=target, parameters=parameters)

        if (parameters["I0"].value is not None) and (
            parameters["aRL"].value is not None
        ):
            return
        target_area = target[self.window]
        icenter = target_area.world_to_pixel(parameters["center"].value)

        if parameters["I0"].value is None:
            parameters["I0"].set_value(
                torch.log10(
                    torch.mean(
                        target_area.data[
                            int(icenter[0]) - 2 : int(icenter[0]) + 2,
                            int(icenter[1]) - 2 : int(icenter[1]) + 2,
                        ]
                    )
                    / target.pixel_area.item()
                ),
                override_locked=True,
            )
            parameters["I0"].set_uncertainty(
                torch.std(
                    target_area.data[
                        int(icenter[0]) - 2 : int(icenter[0]) + 2,
                        int(icenter[1]) - 2 : int(icenter[1]) + 2,
                    ]
                )
                / (torch.abs(parameters["I0"].value) * target.pixel_area),
                override_locked=True,
            )
        if parameters["aRL"].value is None:
            parameters["aRL"].set_value(
                (5./8.) * 2 * target.pixel_length, override_locked=True
            )
            parameters["aRL"].set_uncertainty(parameters["aRL"].value / 2, override_locked=True)
        

    @default_internal
    def radial_model(self, R, image=None, parameters=None):
        x = 2 * torch.pi * parameters["aRL"].value * R

        return (image.pixel_area * 10**parameters["I0"].value) * (2 * torch.special.bessel_j1(x) / x)**2
    
    @default_internal
    def evaluate_model(self, X=None, Y=None, image=None, parameters=None):
        if X is None:
            Coords = image.get_coordinate_meshgrid()
            X, Y = Coords - parameters["center"].value[..., None, None]

        r = self.radius_metric(X, Y, image, parameters)
        return self.radial_model(r, image=image, parameters=parameters)

==> astrophot-0.12.0-py2.py3-none-any.whl: astrophot/models/airy_psf.py <==
import torch
import numpy as np

from ..utils.decorators import ignore_numpy_warnings, default_internal
from ._shared_methods import select_target
from .star_model_object import Star_Model
from .. import AP_config

__all__ = ("Airy_Star",)


class Airy_Star(Star_Model):
    """The Airy disk is an analytic description of the diffraction pattern
    for a circular aperture.

    The diffraction pattern is described exactly by the configuration
    of the lens system under the assumption that all elements are
    perfect. This expression goes as:

    .. math::

      I(\\theta) = I_0\\left[\\frac{2J_1(x)}{x}\\right]^2

      x = ka\\sin(\\theta) = \\frac{2\\pi a r}{\\lambda R}

    where :math:`I(\\theta)` is the intensity as a function of the
    angular poisition within the diffraction system along its main
    axis, :math:`I_0` is the central intensity of the airy disk,
    :math:`J_1` is the Bessel function of the first kind of order one,
    :math:`k = \\frac{2\\pi}{\\lambda}` is the wavenumber of the
    light, :math:`a` is the aperture radius, :math:`r` is the radial
    position from the center of the pattern, :math:`R` is the distance
    from the circular aperture to the observation plane.

    In the `Airy_Star` class we combine the parameters
    :math:`a,R,\\lambda` into a single ratio to be optimized (or fixed
    by the optical configuration).

    """
    model_type = f"airy {Star_Model.model_type}"
    parameter_specs = {
        "I0": {"units": "log10(flux/arcsec^2)"},
        "aRL": {"units": "a/(R lambda)"}
    }
    _parameter_order = Star_Model._parameter_order + ("I0", "aRL")
    useable = True

    @torch.no_grad()
    @ignore_numpy_warnings
    @select_target
    @default_internal
    def initialize(self, target=None, parameters=None, **kwargs):
        super().initialize(target=target, parameters=parameters)

        if (parameters["I0"].value is not None) and (
            parameters["aRL"].value is not None
        ):
            return
        target_area = target[self.window]
        icenter = target_area.plane_to_pixel(parameters["center"].value)

        if parameters["I0"].value is None:
            parameters["I0"].set_value(
                torch.log10(
                    torch.mean(
                        target_area.data[
                            int(icenter[0]) - 2 : int(icenter[0]) + 2,
                            int(icenter[1]) - 2 : int(icenter[1]) + 2,
                        ]
                    )
                    / target.pixel_area.item()
                ),
                override_locked=True,
            )
            parameters["I0"].set_uncertainty(
                torch.std(
                    target_area.data[
                        int(icenter[0]) - 2 : int(icenter[0]) + 2,
                        int(icenter[1]) - 2 : int(icenter[1]) + 2,
                    ]
                )
                / (torch.abs(parameters["I0"].value) * target.pixel_area),
                override_locked=True,
            )
        if parameters["aRL"].value is None:
            parameters["aRL"].set_value(
                (5./8.) * 2 * target.pixel_length, override_locked=True
            )
            parameters["aRL"].set_uncertainty(parameters["aRL"].value / 2, override_locked=True)
        

    @default_internal
    def radial_model(self, R, image=None, parameters=None):
        x = 2 * torch.pi * parameters["aRL"].value * R

        return (image.pixel_area * 10**parameters["I0"].value) * (2 * torch.special.bessel_j1(x) / x)**2
    
    @default_internal
    def evaluate_model(self, X=None, Y=None, image=None, parameters=None):
        if X is None:
            Coords = image.get_coordinate_meshgrid()
            X, Y = Coords - parameters["center"].value[..., None, None]

        r = self.radius_metric(X, Y, image, parameters)
        return self.radial_model(r, image=image, parameters=parameters)
